#define HAVE_MALLOC 1

#include        <gtk/gtk.h>
#include        <stdio.h>
#include        <strings.h>
#include        <dirent.h>
#include 	<errno.h>
#include	<limits.h>
#include	<sys/types.h>
#include        <sys/stat.h>
#include        "trsim.h"
#include 	"ask.h"


#ifndef NAME_MAX
#define NAME_MAX (255 - 1)
#endif

#ifdef HAVE_MALLOC
#include <malloc.h>
#endif

extern char *locale_name;
extern void	load_localized_strings(char *locale);

static	gchar *en_l_titles[] = { "Availables languages" };
static	gchar *l_titles[sizeof(en_l_titles)/sizeof(char *)];


typedef struct {
  char language[80];
  char loc[3]; 
}Locale;

GSList *mylocale;

GtkWidget *dialog_window;

gint compare_func(gconstpointer item1, gconstpointer item2) {
 return g_strcasecmp(item1, item2);
}
    

char *find_loc(const char language[80]) {
	GSList *list_move;
	list_move = mylocale;
	for (list_move = mylocale; list_move; list_move = list_move->next) {
  		if(strcmp(((Locale *)list_move->data)->language,language)==0)
			return ((Locale *)list_move->data)->loc;
 }
}
	

int get_locale(char *flname)
{
    char *str;
    char *str1;
    char buff[256];
    char full[256];
    int n,l;
    FILE *fp;
	n =0;

	g_snprintf(full,sizeof(full),"%s/locales/%s",exepath,flname);
	if(fp=fopen(full,"r")){
	    fgets(buff,sizeof(buff),fp);
	    if(strncmp(buff,"#!",2) == 0) {
	        str=buff+2;
			l = strlen(str);
			if(l && str[l - 1] == '\n') --l;		
				str[l]=0;
#if defined (__unix__)
			str1 = flname + 10;
#else
			str1 = flname + 9;
#endif
			Locale* vlocale = g_new(Locale, 1); // allocate memory for struct
			sprintf(vlocale->language,"%s",str);
			sprintf(vlocale->loc,"%s",str1);
			mylocale = g_slist_append(mylocale,vlocale);  
	    }
		
	return 1;
	} 
	else {
	g_print("error");
	return -1;
	}
}

 int scan_locale (const char *dir) {
   DIR *dp;
   struct dirent *finfo;
   struct stat istat;
   int count = 0;
 
   
   if (dir == NULL)
     return -1;
   
   
	if(stat(dir, &istat) == -1) {
	   g_print("stat %s error %d",dir,errno);
	   return -1;
   }
   
	if(istat.st_mode == S_IFDIR) {
		g_print("%s expected %s to be a directory!\n", __FUNCTION__,dir);
		return -1;
	}
   
	errno=0;
 
   dp = opendir(dir);
   if(( errno == EACCES) || (dp == NULL)) {
	   g_print("Error opendir %s",dir);
	   return -1;
   }
   
   while ((finfo = readdir (dp)) != NULL){
   
   if ((finfo->d_name[0] =='.') && (strlen(finfo->d_name) < 2))
   continue; 
#if defined(__unix__)
   if (strncmp(finfo->d_name,".traindir",7 ) == 0) {
#else
   if (strncmp(finfo->d_name,"traindir",7 ) == 0) {
		
#endif
			if(get_locale(finfo->d_name) == -1) {
				g_print("Error get_locale %s",finfo->d_name);
				closedir(dp);
				return -1;
			}
		}
	count++;
	}
	closedir(dp);
	return count;
}

 /* If we come here, then the user has selected a row in the list. */
void selection_ok( GtkWidget      *clist,
                     gint            row,
                     gint            column,
                     GdkEventButton *event,
                     gpointer        data )
{
    gchar *text;

    /* Get the text that is stored in the selected row and column
     * which was clicked in. We will receive it as a pointer in the
     * argument text.
     */
    gtk_clist_get_text(GTK_CLIST(clist), row, column, &text);
	locale_name = find_loc(text);
	load_localized_strings(locale_name);
    return;
}




gboolean button_ok_cb(GtkWidget *widget) {
	
	if(ask(L("Write selected language to ini file?\n You must restart program to apply change"))){
	    save_prefs = 1;
	    save_ini_file();
	    gtk_widget_destroy(dialog_window);
		return TRUE;
	}
return FALSE;
}


void gtk_locale_dialog()
{   	

    GtkWidget *vbox, *hbox;
    GtkWidget *scrolled_window, *cliste;
    GtkWidget *button_cancel, *button_ok;    
    l_titles[0] = L(en_l_titles[0]);
    Locale *vlocale;
    GSList *locale_move = NULL;
    char dir[NAME_MAX];

    /* default language*/
    mylocale = g_slist_alloc();
    if((vlocale = (Locale *)malloc(sizeof(Locale))) == NULL)
  { perror("malloc failed "); exit(1); }
    sprintf(vlocale->language,"English");
    sprintf(vlocale->loc,"en");
    mylocale = g_slist_prepend(mylocale,vlocale);  
    /* Search for installed language file */
    g_snprintf(dir,sizeof(dir),"%s/locales",exepath);
    scan_locale(dir);
    g_slist_remove(mylocale, NULL);
    mylocale = g_slist_sort(mylocale, (GCompareFunc)compare_func);
    dialog_window=gtk_window_new(GTK_WINDOW_DIALOG);
    gtk_widget_set_usize(GTK_WIDGET(dialog_window), 300, 200);
    gtk_window_set_modal(GTK_WINDOW(dialog_window),TRUE);

    gtk_window_set_title(GTK_WINDOW(dialog_window), L("Language selection"));
    gtk_signal_connect(GTK_OBJECT(dialog_window),
                       "destroy",
                       GTK_SIGNAL_FUNC(gtk_widget_destroy),
                       NULL);
    
    vbox=gtk_vbox_new(FALSE, 5);
    gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
    gtk_container_add(GTK_CONTAINER(dialog_window), vbox);
    gtk_widget_show(vbox);
    
    /* Create a scrolled window to pack the CList widget into */
    scrolled_window = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);

    gtk_box_pack_start(GTK_BOX(vbox), scrolled_window, TRUE, TRUE, 4);
    gtk_widget_show (scrolled_window);

    /* Create the CList. For this example we use 2 columns */
    cliste = gtk_clist_new_with_titles( 1, l_titles);

    /* When a selection is made, we want to know about it. The callback
     * used is selection_made, and its code can be found further down */
    gtk_signal_connect(GTK_OBJECT(cliste), "select_row",
                       GTK_SIGNAL_FUNC(selection_ok),
                       NULL);

    /* It isn't necessary to shadow the border, but it looks nice :) */
    gtk_clist_set_shadow_type (GTK_CLIST(cliste), GTK_SHADOW_OUT);

    /* What however is important, is that we set the column widths as
     * they will never be right otherwise. Note that the columns are
     * numbered from 0 and up (to 1 in this case).
     */
    gtk_clist_set_column_width (GTK_CLIST(cliste), 0, 80);
	gtk_clist_set_selection_mode(GTK_CLIST(cliste),GTK_SELECTION_SINGLE);

    /* Add the CList widget to the vertical box and show it. */
    gtk_container_add(GTK_CONTAINER(scrolled_window), cliste);
	char language[80];
	char *p;
	gtk_clist_freeze(GTK_CLIST(cliste));
	gtk_clist_clear(GTK_CLIST(cliste));
	for(locale_move=mylocale;locale_move;locale_move=locale_move->next) {
		sprintf(language,"%s",((Locale *)locale_move->data)->language);
		//g_print("Langage %s",language);
		p=language;
		gtk_clist_append(GTK_CLIST(cliste),&p);
		}
	gtk_clist_thaw(GTK_CLIST(cliste));
    gtk_widget_show(cliste);

    /* Create the buttons and add them to the window. See the button
     * tutorial for more examples and comments on this.
     */
    hbox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 4);
    gtk_widget_show(hbox);

    button_cancel = gtk_button_new_with_label(L("Cancel"));
    button_ok = gtk_button_new_with_label(L("Ok"));
	GTK_WIDGET_SET_FLAGS (button_cancel, GTK_CAN_DEFAULT);

    gtk_box_pack_start(GTK_BOX(hbox), button_cancel, TRUE, TRUE, 4);
    gtk_box_pack_start(GTK_BOX(hbox), button_ok, TRUE, TRUE, 4);

    /* Connect our callbacks to the three buttons */
    gtk_signal_connect_object(GTK_OBJECT(button_cancel), "clicked",
                              GTK_SIGNAL_FUNC(gtk_widget_destroy),
							  (gpointer) dialog_window);
							  
    gtk_signal_connect_object(GTK_OBJECT(button_ok), "clicked",
                              GTK_SIGNAL_FUNC(button_ok_cb),
                              (gpointer) cliste);

    gtk_widget_show(button_cancel);
    gtk_widget_show(button_ok);

    /* The interface is completely set up so we show the window 
     */
    gtk_widget_show(dialog_window);
    
	return;
}
